==> acquia_perz-4.0.0-rc2.tar.gz: acquia_perz/src/Plugin/GraphQL/DataProducer/Entity/QueryDiscoverEntities.php <==
<?php

namespace Drupal\acquia_perz\Plugin\GraphQL\DataProducer\Entity;

use Drupal\acquia_perz\EntityHelper;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Discover entitites producer.
 *
 * @DataProducer(
 *   id = "query_discover_entities",
 *   name = @Translation("Discover all available entities"),
 *   description = @Translation("Return a list of all available entities of different entity types."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("List of entities")
 *   ),
 *   consumes = {
 *     "page" = @ContextDefinition("integer",
 *       label = @Translation("Pagination: page number")
 *     )
 *   }
 * )
 */
class QueryDiscoverEntities extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The acquia perz entity config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $entityConfig;

  /**
   * The CIS settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $cisSettings;

  /**
   * The Perz entity helper.
   *
   * @var \Drupal\acquia_perz\EntityHelper
   */
  protected $entityHelper;

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('acquia_perz.entity_helper')
    );
  }

  /**
   * Constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\acquia_perz\EntityHelper $entity_helper
   *   The entity helper.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    EntityHelper $entity_helper
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->entityConfig = $config_factory->get('acquia_perz.entity_config');
    $this->cisSettings = $config_factory->get('acquia_perz.settings');
    $this->entityTypeManager = $entity_type_manager;
    $this->entityHelper = $entity_helper;
  }

  /**
   * Resolver.
   *
   * @param int $page
   *   Pagination: current page number.
   * @param \Drupal\Core\Cache\RefinableCacheableDependencyInterface $metadata
   *   The metadata.
   *
   * @return array
   *   Response array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function resolve($page, RefinableCacheableDependencyInterface $metadata) {
    $entity_items = [];
    // Number of entities that can be present on one page.
    $page_max_size = $this->cisSettings->get('api.discovery_endpoint_page_size');
    // Global offset for entities of all available
    // entity types.
    // It will be updated within each iteration.
    $offset = $page_max_size * $page;
    // Global limit for entities of all available
    // entity types.
    // It will be updated within each iteration.
    $limit = $page_max_size;
    $entity_types = $this->entityConfig->get('view_modes');
    if (empty($entity_types)) {
      throw new \Exception("No entity types are available for the export.");
    }
    foreach ($entity_types as $entity_type_id => $bundles) {
      // Number of all available entities by entity type.
      $et_all_entities_count = $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles);
      // Number of available entities by entity type
      // that restricted by current Offset and Limit
      // parameters.
      $et_left_entities_count = $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles, $offset, $limit);
      $entity_type = $this->entityTypeManager
        ->getStorage($entity_type_id)
        ->getEntityType();
      // This means that all entities of current entity type
      // is not available because of offset. So skip current
      // entity type in this case and decrease offset value
      // that will be used for next entity type's iterations.
      if (!$et_left_entities_count) {
        $offset = ($offset > $et_all_entities_count) ?
          $offset - $et_all_entities_count : 0;
        continue;
      }

      // Get entities by current entity type, its bundles,
      // global limit and global offset.
      $entity_items = array_merge(
        $this->getEntitiesUuids(
          $entity_type_id,
          $bundles,
          $offset,
          $limit
        ),
        $entity_items
      );

      // Update global offset and limit.
      $offset = ($offset > $et_all_entities_count) ?
        $offset - $et_all_entities_count : 0;
      $limit = $limit - $et_left_entities_count;

      // Stop searching new entities if limit is
      // not positive.
      if ($limit <= 0) {
        break;
      }

      $metadata->addCacheTags($entity_type->getListCacheTags());
      $metadata->addCacheContexts($entity_type->getListCacheContexts());
    }
    $metadata->addCacheTags($this->entityConfig->getCacheTags());
    $metadata->addCacheContexts($this->entityConfig->getCacheContexts());
    $metadata->addCacheTags($this->cisSettings->getCacheTags());
    $metadata->addCacheContexts($this->cisSettings->getCacheContexts());

    // Calculate Page info payload.
    $total_count = $this->getTotalCount($entity_types);
    $pages_number_raw = ($total_count / $page_max_size) - 1;
    $pages_number_rounded = floor($pages_number_raw);
    $pages_number = $pages_number_rounded;
    if ($pages_number_rounded < $pages_number_raw) {
      ++$pages_number;
    }

    return [
      'page_info' => [
        'total_count' => $total_count,
        'current_page_count' => count($entity_items),
        'current_page' => $page,
        'next_page' => ($page < $pages_number) ? $page + 1 : NULL,
        'prev_page' => ($page > 0) ? $page - 1 : NULL,
      ],
      'items' => $entity_items,
    ];
  }

  /**
   * Get list of entity uuids.
   *
   * Format:
   * [
   *   [
   *     'entity_type_id' => '...',
   *     'entity_uuid' => '...',
   *   ]
   * ].
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns a list of entities by its entity type id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesUuids($entity_type_id, array $bundles, $offset, $limit) {
    $entity_ids = $this->getEntitiesIds($entity_type_id, $bundles, $offset, $limit);
    $entity_type = $this->entityTypeManager
      ->getStorage($entity_type_id)
      ->getEntityType();
    $entity_items = [];
    foreach ($entity_ids as $entity_id) {
      $entity_uuid = $this->entityHelper->getEntityUuidById($entity_type, $entity_id);
      $entity_items[] = [
        'entity_type_id' => $entity_type_id,
        'entity_uuid' => $entity_uuid,
      ];
    }
    return $entity_items;
  }

  /**
   * Get entity ids by entity type id and passed bundles.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns entity ids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesIds($entity_type_id, array $bundles, int $offset = NULL, int $limit = NULL) {
    $query = $this->entityHelper->getEntitiesQuery($entity_type_id, $bundles, $offset, $limit);
    return array_values($query->execute());
  }

  /**
   * Get number of all entities by its entity types.
   *
   * @param array $entity_types
   *   List of entity types.
   *
   * @return int
   *   Returns number of all available entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getTotalCount(array $entity_types) {
    $total_count = 0;
    foreach ($entity_types as $entity_type_id => $bundles) {
      $total_count += $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles);
    }
    return $total_count;
  }

}

==> acquia_perz-4.1.1.tar.gz: acquia_perz/src/Plugin/GraphQL/DataProducer/Entity/QueryDiscoverEntities.php <==
<?php

namespace Drupal\acquia_perz\Plugin\GraphQL\DataProducer\Entity;

use Drupal\acquia_perz\EntityHelper;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Discover entitites producer.
 *
 * @DataProducer(
 *   id = "query_discover_entities",
 *   name = @Translation("Discover all available entities"),
 *   description = @Translation("Return a list of all available entities of different entity types."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("List of entities")
 *   ),
 *   consumes = {
 *     "page" = @ContextDefinition("integer",
 *       label = @Translation("Pagination: page number")
 *     )
 *   }
 * )
 */
class QueryDiscoverEntities extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The acquia perz entity config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $entityConfig;

  /**
   * The CIS settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $cisSettings;

  /**
   * The Perz entity helper.
   *
   * @var \Drupal\acquia_perz\EntityHelper
   */
  protected $entityHelper;

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('acquia_perz.entity_helper')
    );
  }

  /**
   * Constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\acquia_perz\EntityHelper $entity_helper
   *   The entity helper.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    EntityHelper $entity_helper
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->entityConfig = $config_factory->get('acquia_perz.entity_config');
    $this->cisSettings = $config_factory->get('acquia_perz.settings');
    $this->entityTypeManager = $entity_type_manager;
    $this->entityHelper = $entity_helper;
  }

  /**
   * Resolver.
   *
   * @param int $page
   *   Pagination: current page number.
   * @param \Drupal\Core\Cache\RefinableCacheableDependencyInterface $metadata
   *   The metadata.
   *
   * @return array
   *   Response array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function resolve($page, RefinableCacheableDependencyInterface $metadata) {
    $entity_items = [];
    // Number of entities that can be present on one page.
    $page_max_size = $this->cisSettings->get('api.discovery_endpoint_page_size');
    // Global offset for entities of all available
    // entity types.
    // It will be updated within each iteration.
    $offset = $page_max_size * $page;
    // Global limit for entities of all available
    // entity types.
    // It will be updated within each iteration.
    $limit = $page_max_size;
    $entity_types = $this->entityConfig->get('view_modes');
    if (empty($entity_types)) {
      throw new \Exception("No entity types are available for the export.");
    }
    foreach ($entity_types as $entity_type_id => $bundles) {
      // Number of all available entities by entity type.
      $et_all_entities_count = $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles);
      // Number of available entities by entity type
      // that restricted by current Offset and Limit
      // parameters.
      $et_left_entities_count = $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles, $offset, $limit);
      $entity_type = $this->entityTypeManager
        ->getStorage($entity_type_id)
        ->getEntityType();
      // This means that all entities of current entity type
      // is not available because of offset. So skip current
      // entity type in this case and decrease offset value
      // that will be used for next entity type's iterations.
      if (!$et_left_entities_count) {
        $offset = ($offset > $et_all_entities_count) ?
          $offset - $et_all_entities_count : 0;
        continue;
      }

      // Get entities by current entity type, its bundles,
      // global limit and global offset.
      $entity_items = array_merge(
        $this->getEntitiesUuids(
          $entity_type_id,
          $bundles,
          $offset,
          $limit
        ),
        $entity_items
      );

      // Update global offset and limit.
      $offset = ($offset > $et_all_entities_count) ?
        $offset - $et_all_entities_count : 0;
      $limit = $limit - $et_left_entities_count;

      // Stop searching new entities if limit is
      // not positive.
      if ($limit <= 0) {
        break;
      }

      $metadata->addCacheTags($entity_type->getListCacheTags());
      $metadata->addCacheContexts($entity_type->getListCacheContexts());
    }
    $metadata->addCacheTags($this->entityConfig->getCacheTags());
    $metadata->addCacheContexts($this->entityConfig->getCacheContexts());
    $metadata->addCacheTags($this->cisSettings->getCacheTags());
    $metadata->addCacheContexts($this->cisSettings->getCacheContexts());

    // Calculate Page info payload.
    $total_count = $this->getTotalCount($entity_types);
    $pages_number_raw = ($total_count / $page_max_size) - 1;
    $pages_number_rounded = floor($pages_number_raw);
    $pages_number = $pages_number_rounded;
    if ($pages_number_rounded < $pages_number_raw) {
      ++$pages_number;
    }

    return [
      'page_info' => [
        'total_count' => $total_count,
        'current_page_count' => count($entity_items),
        'current_page' => $page,
        'next_page' => ($page < $pages_number) ? $page + 1 : NULL,
        'prev_page' => ($page > 0) ? $page - 1 : NULL,
      ],
      'items' => $entity_items,
    ];
  }

  /**
   * Get list of entity uuids.
   *
   * Format:
   * [
   *   [
   *     'entity_type_id' => '...',
   *     'entity_uuid' => '...',
   *   ]
   * ].
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns a list of entities by its entity type id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesUuids($entity_type_id, array $bundles, $offset, $limit) {
    $entity_ids = $this->getEntitiesIds($entity_type_id, $bundles, $offset, $limit);
    $entity_type = $this->entityTypeManager
      ->getStorage($entity_type_id)
      ->getEntityType();
    $entity_items = [];
    foreach ($entity_ids as $entity_id) {
      $entity_uuid = $this->entityHelper->getEntityUuidById($entity_type, $entity_id);
      $entity_items[] = [
        'entity_type_id' => $entity_type_id,
        'entity_uuid' => $entity_uuid,
      ];
    }
    return $entity_items;
  }

  /**
   * Get entity ids by entity type id and passed bundles.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns entity ids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesIds($entity_type_id, array $bundles, int $offset = NULL, int $limit = NULL) {
    $query = $this->entityHelper->getEntitiesQuery($entity_type_id, $bundles, $offset, $limit);
    return array_values($query->accessCheck(TRUE)->execute());
  }

  /**
   * Get number of all entities by its entity types.
   *
   * @param array $entity_types
   *   List of entity types.
   *
   * @return int
   *   Returns number of all available entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getTotalCount(array $entity_types) {
    $total_count = 0;
    foreach ($entity_types as $entity_type_id => $bundles) {
      $total_count += $this->entityHelper->getCountByEntityTypeId($entity_type_id, $bundles);
    }
    return $total_count;
  }

}
